# [General]
# [Replica]
# [Proxy]
# [Proxy Group]
# [Rule]
# [Host]
# [URL Rewrite]
# [Script]
# [MITM]

from typing import Literal


SurgeConfigSections = Literal[
    'General',
    'Replica',
    'Proxy',
    'Proxy Group',
    'Rule',
    'Host',
    'URL Rewrite',
    'Script',
    'MITM',
]
